@@ -203,10 +203,12 @@ function get_lhs_rhs(@nospecialize stmt)
     if isexpr(stmt, :(=))
         return Pair{Any,Any}(stmt.args[1], stmt.args[2])
     elseif isexpr(stmt, :const) && length(stmt.args) == 2
+        # TODO: remove lowered :const when Julia min compat >= 1.13
         return Pair{Any,Any}(stmt.args[1], stmt.args[2])
     elseif isexpr(stmt, :call) && length(stmt.args) == 4
         f = stmt.args[1]
-        if is_global_ref_egal(f, :setglobal!, Core.setglobal!)
+        # TODO: remove isdefined when Julia min compat >= 1.13
+        if is_global_ref_egal(f, :setglobal!, Core.setglobal!) || @static isdefined(Core, :declare_const) && is_global_ref_egal(f, :declare_const,  Core.declare_const)
             mod = stmt.args[2]
             mod isa Module || return nothing
             name = stmt.args[3]
@@ -242,27 +244,24 @@ function direct_links!(cl::CodeLinks, src::CodeInfo)
             add_inner!(cl, icl, i)
             continue
         elseif isexpr(stmt, :method)
-            if length(stmt.args) === 3 && (arg3 = stmt.args[3]; arg3 isa CodeInfo)
-                icl = CodeLinks(cl.thismod, arg3)
-                add_inner!(cl, icl, i)
-            end
-            name = stmt.args[1]
-            if isa(name, GlobalRef) || isa(name, Symbol)
+            if length(stmt.args) === 1
+                # A function with no methods was defined. Associate its new binding to it.
+                name = stmt.args[1]
                 if isa(name, Symbol)
                     name = GlobalRef(cl.thismod, name)
                 end
-                assign = get(cl.nameassigns, name, nothing)
-                if assign === nothing
-                    cl.nameassigns[name] = assign = Int[]
+                if !isa(name, GlobalRef)
+                    error("name ", typeof(name), " not recognized")
                 end
+                assign = get!(Vector{Int}, cl.nameassigns, name)
                 push!(assign, i)
                 targetstore = get!(Links, cl.namepreds, name)
                 target = P(name, targetstore)
                 add_links!(target, stmt, cl)
-            elseif name in (nothing, false)
-            else
-                @show stmt
-                error("name ", typeof(name), " not recognized")
+            elseif length(stmt.args) === 3 && (arg3 = stmt.args[3]; arg3 isa CodeInfo) # method definition
+                # A method was defined for an existing function.
+                icl = CodeLinks(cl.thismod, arg3)
+                add_inner!(cl, icl, i)
             end
             rhs = stmt
             target = P(SSAValue(i), cl.ssapreds[i])
@@ -340,6 +339,21 @@ function add_links!(target::Pair{Union{SSAValue,SlotNumber,GlobalRef},Links}, @n
                     # Avoid putting named callees on the namestore
                     arng = 2:length(stmt.args)
                 end
+                if f isa GlobalRef && f == GlobalRef(Core, :declare_global) && 3 <= length(stmt.args) <= 5
+                    # Core.declare_global(module::Module, name::Symbol, strong::Bool=false, [ty::Type])
+                    m = stmt.args[2]
+                    s = stmt.args[3]
+                    strong = length(stmt.args) >= 4 ? stmt.args[4] === true : false
+                    if strong && m isa Module && s isa QuoteNode && s.value isa Symbol
+                        a = GlobalRef(m, s.value::Symbol)
+                        namestore = get!(Links, cl.namepreds, a) # TODO should this information be tracked in the separate `cl.namedecls` store?
+                        push!(namestore, targetid)
+                        if targetid isa SSAValue
+                            push!(namestore, SSAValue(targetid.id+1)) # +1 for :latestworld
+                        end
+                    end
+                    arng = 4:length(stmt.args)
+                end
             end
             for i in arng
                 add_links!(target, stmt.args[i], cl)
